/*
 * Copyright (C) 2012 by
 *   MetraLabs GmbH (MLAB), GERMANY
 * and
 *   Neuroinformatics and Cognitive Robotics Labs (NICR) at TU Ilmenau, GERMANY
 * All rights reserved.
 *
 * Contact: info@mira-project.org
 *
 * Commercial Usage:
 *   Licensees holding valid commercial licenses may use this file in
 *   accordance with the commercial license agreement provided with the
 *   software or, alternatively, in accordance with the terms contained in
 *   a written agreement between you and MLAB or NICR.
 *
 * GNU General Public License Usage:
 *   Alternatively, this file may be used under the terms of the GNU
 *   General Public License version 3.0 as published by the Free Software
 *   Foundation and appearing in the file LICENSE.GPL3 included in the
 *   packaging of this file. Please review the following information to
 *   ensure the GNU General Public License version 3.0 requirements will be
 *   met: http://www.gnu.org/copyleft/gpl.html.
 *   Alternatively you may (at your option) use any later version of the GNU
 *   General Public License if such license has been publicly approved by
 *   MLAB and NICR (or its successors, if any).
 *
 * IN NO EVENT SHALL "MLAB" OR "NICR" BE LIABLE TO ANY PARTY FOR DIRECT,
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
 * THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF "MLAB" OR
 * "NICR" HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * "MLAB" AND "NICR" SPECIFICALLY DISCLAIM ANY WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND "MLAB" AND "NICR" HAVE NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS OR MODIFICATIONS.
 */

/**
 * @file OgreWidget.h
 *    Declaration of OgreWidget and related classes.
 *
 * @author Erik Einhorn
 * @date   2010/12/02
 */

#ifndef _MIRA_OGREWIDGET_H_
#define _MIRA_OGREWIDGET_H_

#include <QGLWidget>

#include <widgets/GuiWidgetsExports.h>

///////////////////////////////////////////////////////////////////////////////

namespace Ogre {
	class RenderWindow;
}

///////////////////////////////////////////////////////////////////////////////

namespace mira {

///////////////////////////////////////////////////////////////////////////////

class OgreSingletonHelper : public QObject
{
	Q_OBJECT

public:
	 ~OgreSingletonHelper() {}

public slots:
	virtual void onAboutToQuit() = 0;
};

///////////////////////////////////////////////////////////////////////////////

class MIRA_GUI_WIDGETS_EXPORT OgreWidget : public QGLWidget
{
public:

	explicit OgreWidget(QWidget* parent=0);
	virtual ~OgreWidget();

public:
	virtual void paintGL();

protected:
	/**
	 * May be overwritten in subclasses to initialize your Ogre components.
	 * This method is called after the main parts of Ogre (including the
	 * render window) are created.
	 */
	virtual void initOgre() {}

protected:
	// overwritten from QGLWidget
	virtual void initializeGL();
	virtual void resizeGL(int w, int h);

protected:

	Ogre::RenderWindow* getRenderWindow() { return mRenderWindow; }

private:
	Ogre::RenderWindow* mRenderWindow;
};

///////////////////////////////////////////////////////////////////////////////

}

#endif
